@@ -22,15 +22,17 @@
 # 2007-09-08 SJM Work around corrupt STYLE record
 # 2007-07-11 SJM Allow for BIFF2/3-style FORMAT record in BIFF4/8 file
 
+from __future__ import print_function
+
 DEBUG = 0
 import copy, re
-from timemachine import *
-from biffh import BaseObject, unpack_unicode, unpack_string, \
+from struct import unpack
+from .timemachine import *
+from .biffh import BaseObject, unpack_unicode, unpack_string, \
     upkbits, upkbitsL, fprintf, \
     FUN, FDT, FNU, FGE, FTX, XL_CELL_NUMBER, XL_CELL_DATE, \
     XL_FORMAT, XL_FORMAT2, \
     XLRDError
-from struct import unpack
 
 excel_default_palette_b5 = (
     (  0,   0,   0), (255, 255, 255), (255,   0,   0), (  0, 255,   0),
@@ -149,8 +151,8 @@ def nearest_colour_index(colour_map, rgb, debug=0):
             if metric == 0:
                 break
     if 0 and debug:
-        print "nearest_colour_index for %r is %r -> %r; best_metric is %d" \
-            % (rgb, best_colourx, colour_map[best_colourx], best_metric)
+        print("nearest_colour_index for %r is %r -> %r; best_metric is %d" \
+            % (rgb, best_colourx, colour_map[best_colourx], best_metric))
     return best_colourx
 
 ##
@@ -219,7 +221,7 @@ class Font(BaseObject, EqNeAttrs):
     italic = 0
     ##
     # The name of the font. Example: u"Arial"
-    name = u""
+    name = UNICODE_LITERAL("")
     ##
     # 1 = Characters are struck out.
     struck_out = 0
@@ -259,7 +261,7 @@ def handle_font(book, data):
     k = len(book.font_list)
     if k == 4:
         f = Font()
-        f.name = u'Dummy Font'
+        f.name = UNICODE_LITERAL('Dummy Font')
         f.font_index = k
         book.font_list.append(f)
         k += 1
@@ -341,7 +343,7 @@ class Format(BaseObject, EqNeAttrs):
     type = FUN
     ##
     # The format string
-    format_str = u''
+    format_str = UNICODE_LITERAL('')
 
     def __init__(self, format_key, ty, format_str):
         self.format_key = format_key
@@ -414,29 +416,29 @@ for lo, hi, ty in fmt_code_ranges:
         std_format_code_types[x] = ty
 del lo, hi, ty, x
 
-date_chars = u'ymdhs' # year, month/minute, day, hour, second
+date_chars = UNICODE_LITERAL('ymdhs') # year, month/minute, day, hour, second
 date_char_dict = {}
 for _c in date_chars + date_chars.upper():
     date_char_dict[_c] = 5
 del _c, date_chars
 
 skip_char_dict = {}
-for _c in u'$-+/(): ':
+for _c in UNICODE_LITERAL('$-+/(): '):
     skip_char_dict[_c] = 1
 
 num_char_dict = {
-    u'0': 5,
-    u'#': 5,
-    u'?': 5,
+    UNICODE_LITERAL('0'): 5,
+    UNICODE_LITERAL('#'): 5,
+    UNICODE_LITERAL('?'): 5,
     }
 
 non_date_formats = {
-    u'0.00E+00':1,
-    u'##0.0E+0':1,
-    u'General' :1,
-    u'GENERAL' :1, # OOo Calc 1.1.4 does this.
-    u'general' :1,  # pyExcelerator 0.6.3 does this.
-    u'@'       :1,
+    UNICODE_LITERAL('0.00E+00'):1,
+    UNICODE_LITERAL('##0.0E+0'):1,
+    UNICODE_LITERAL('General') :1,
+    UNICODE_LITERAL('GENERAL') :1, # OOo Calc 1.1.4 does this.
+    UNICODE_LITERAL('general') :1,  # pyExcelerator 0.6.3 does this.
+    UNICODE_LITERAL('@')       :1,
     }
 
 fmt_bracketed_sub = re.compile(r'\[[^]]*\]').sub
@@ -458,37 +460,37 @@ def is_date_format_string(book, fmt):
     # TODO: u'[h]\\ \\h\\o\\u\\r\\s' ([h] means don't care about hours > 23)
     state = 0
     s = ''
-    ignorable = skip_char_dict.has_key
+    
     for c in fmt:
         if state == 0:
-            if c == u'"':
+            if c == UNICODE_LITERAL('"'):
                 state = 1
-            elif c in ur"\_*":
+            elif c in UNICODE_LITERAL(r"\_*"):
                 state = 2
-            elif ignorable(c):
+            elif c in skip_char_dict:
                 pass
             else:
                 s += c
         elif state == 1:
-            if c == u'"':
+            if c == UNICODE_LITERAL('"'):
                 state = 0
         elif state == 2:
             # Ignore char after backslash, underscore or asterisk
             state = 0
         assert 0 <= state <= 2
     if book.verbosity >= 4:
-        print >> book.logfile, "is_date_format_string: reduced format is %r" % s
+        print("is_date_format_string: reduced format is %r" % s, file=book.logfile)
     s = fmt_bracketed_sub('', s)
-    if non_date_formats.has_key(s):
+    if s in non_date_formats:
         return False
     state = 0
     separator = ";"
     got_sep = 0
     date_count = num_count = 0
     for c in s:
-        if date_char_dict.has_key(c):
+        if c in date_char_dict:
             date_count += date_char_dict[c]
-        elif num_char_dict.has_key(c):
+        elif c in num_char_dict:
             num_count += num_char_dict[c]
         elif c == separator:
             got_sep = 1
@@ -596,7 +598,7 @@ def handle_palette(book, data):
         book.colour_map[8+i] = new_rgb
         if blah:
             if new_rgb != old_rgb:
-                print >> book.logfile, "%2d: %r -> %r" % (i, old_rgb, new_rgb)
+                print("%2d: %r -> %r" % (i, old_rgb, new_rgb), file=book.logfile)
 
 def palette_epilogue(book):
     # Check colour indexes in fonts etc.
@@ -608,17 +610,15 @@ def palette_epilogue(book):
         cx = font.colour_index
         if cx == 0x7fff: # system window text colour
             continue
-        if book.colour_map.has_key(cx):
+        if cx in book.colour_map:
             book.colour_indexes_used[cx] = 1
         elif book.verbosity:
-            print >> book.logfile, "Size of colour table:", len(book.colour_map)
-            print >> book.logfile, \
-                "*** Font #%d (%r): colour index 0x%04x is unknown" \
-                % (font.font_index, font.name, cx)
+            print("Size of colour table:", len(book.colour_map), file=book.logfile)
+            print("*** Font #%d (%r): colour index 0x%04x is unknown" \
+                % (font.font_index, font.name, cx), file=book.logfile)
     if book.verbosity >= 1:
-        used = book.colour_indexes_used.keys()
-        used.sort()
-        print >> book.logfile, "\nColour indexes used:\n%r\n" % used
+        used = sorted(book.colour_indexes_used.keys())
+        print("\nColour indexes used:\n%r\n" % used, file=book.logfile)
 
 def handle_style(book, data):
     if not book.formatting_info:
@@ -627,7 +627,7 @@ def handle_style(book, data):
     bv = book.biff_version
     flag_and_xfx, built_in_id, level = unpack('<HBB', data[:4])
     xf_index = flag_and_xfx & 0x0fff
-    if (data == "\0\0\0\0"
+    if (data == b"\0\0\0\0"
     and "Normal" not in book.style_name_map):
         # Erroneous record (doesn't have built-in bit set).
         # Example file supplied by Jeff Bell.
@@ -651,40 +651,35 @@ def handle_style(book, data):
             try:
                 name = unpack_unicode(data, 2, lenlen=2)
             except UnicodeDecodeError:
-                print >> book.logfile, \
-                    "STYLE: built_in=%d xf_index=%d built_in_id=%d level=%d" \
-                    % (built_in, xf_index, built_in_id, level)
-                print >> book.logfile, "raw bytes:", repr(data[2:])
+                print("STYLE: built_in=%d xf_index=%d built_in_id=%d level=%d" \
+                    % (built_in, xf_index, built_in_id, level), file=book.logfile)
+                print("raw bytes:", repr(data[2:]), file=book.logfile)
                 raise
         else:
             name = unpack_string(data, 2, book.encoding, lenlen=1)
         if blah and not name:
-            print >> book.logfile, \
-                "WARNING *** A user-defined style has a zero-length name"
+            print("WARNING *** A user-defined style has a zero-length name", file=book.logfile)
     book.style_name_map[name] = (built_in, xf_index)
     if blah:
-        print >> book.logfile, \
-            "STYLE: built_in=%d xf_index=%d built_in_id=%d level=%d name=%r" \
-            % (built_in, xf_index, built_in_id, level, name)
+        print("STYLE: built_in=%d xf_index=%d built_in_id=%d level=%d name=%r" \
+            % (built_in, xf_index, built_in_id, level, name), file=book.logfile)
 
 def check_colour_indexes_in_obj(book, obj, orig_index):
-    alist = obj.__dict__.items()
-    alist.sort()
+    alist = sorted(obj.__dict__.items())
     for attr, nobj in alist:
         if hasattr(nobj, 'dump'):
             check_colour_indexes_in_obj(book, nobj, orig_index)
         elif attr.find('colour_index') >= 0:
-            if book.colour_map.has_key(nobj):
+            if nobj in book.colour_map:
                 book.colour_indexes_used[nobj] = 1
                 continue
             oname = obj.__class__.__name__
-            print >> book.logfile, \
-                "*** xf #%d : %s.%s =  0x%04x (unknown)" \
-                % (orig_index, oname, attr, nobj)
+            print("*** xf #%d : %s.%s =  0x%04x (unknown)" \
+                % (orig_index, oname, attr, nobj), file=book.logfile)
 
 def fill_in_standard_formats(book):
     for x in std_format_code_types.keys():
-        if not book.format_map.has_key(x):
+        if x not in book.format_map:
             ty = std_format_code_types[x]
             # Note: many standard format codes (mostly CJK date formats) have
             # format strings that vary by locale; xlrd does not (yet)
@@ -756,7 +751,7 @@ def handle_xf(self, data):
             (16, 0x007f0000,  'left_colour_index'),
             (23, 0x3f800000,  'right_colour_index'),
             (30, 0x40000000,  'diag_down'),
-            (31, 0x80000000L, 'diag_up'),
+            (31, 0x80000000, 'diag_up'),
             ))
         upkbits(xf.border, pkd_brdbkg2, (
             (0,  0x0000007F, 'top_colour_index'),
@@ -765,7 +760,7 @@ def handle_xf(self, data):
             (21, 0x01E00000, 'diag_line_style'),
             ))
         upkbitsL(xf.background, pkd_brdbkg2, (
-            (26, 0xFC000000L, 'fill_pattern'),
+            (26, 0xFC000000, 'fill_pattern'),
             ))
         upkbits(xf.background, pkd_brdbkg3, (
             (0, 0x007F, 'pattern_colour_index'),
@@ -806,7 +801,7 @@ def handle_xf(self, data):
             ))
         upkbitsL(xf.border, pkd_brdbkg1, (
             (22, 0x01C00000,  'bottom_line_style'),
-            (25, 0xFE000000L, 'bottom_colour_index'),
+            (25, 0xFE000000, 'bottom_colour_index'),
             ))
         upkbits(xf.border, pkd_brdbkg2, (
             ( 0, 0x00000007, 'top_line_style'),
@@ -857,7 +852,7 @@ def handle_xf(self, data):
             (16, 0x00070000,  'bottom_line_style'),
             (19, 0x00F80000,  'bottom_colour_index'),
             (24, 0x07000000,  'right_line_style'),
-            (27, 0xF8000000L, 'right_colour_index'),
+            (27, 0xF8000000, 'right_colour_index'),
             ))
     elif bv == 30:
         unpack_fmt = '<BBBBHHI'
@@ -899,7 +894,7 @@ def handle_xf(self, data):
             (16, 0x00070000,  'bottom_line_style'),
             (19, 0x00F80000,  'bottom_colour_index'),
             (24, 0x07000000,  'right_line_style'),
-            (27, 0xF8000000L, 'right_colour_index'),
+            (27, 0xF8000000, 'right_colour_index'),
             ))
         xf.alignment.vert_align = 2 # bottom
         xf.alignment.rotation = 0
@@ -956,7 +951,7 @@ def handle_xf(self, data):
             msg = "WARNING *** XF[%d] is a style XF but parent_style_index is 0x%04x, not 0x0fff\n"
             fprintf(self.logfile, msg, xf.xf_index, xf.parent_style_index)
         check_colour_indexes_in_obj(self, xf, xf.xf_index)
-    if not self.format_map.has_key(xf.format_key):
+    if xf.format_key not in self.format_map:
         msg = "WARNING *** XF[%d] unknown (raw) format key (%d, 0x%04x)\n"
         if self.verbosity:
             fprintf(self.logfile, msg,
@@ -981,7 +976,7 @@ def xf_epilogue(self):
 
     for xfx in xrange(num_xfs):
         xf = self.xf_list[xfx]
-        if not self.format_map.has_key(xf.format_key):
+        if xf.format_key not in self.format_map:
             msg = "ERROR *** XF[%d] unknown format key (%d, 0x%04x)\n"
             fprintf(self.logfile, msg,
                     xf.xf_index, xf.format_key, xf.format_key)
